/*
 * SPI.c
 *
 *  Created on: Mar 26, 2019
 *      Author: rhoehne
 */



#include "SPI.h"


/* Global variables */
ARM_DRIVER_SPI *spi0, *spi1;
uint8_t spi0_rx_buffer[SPI_BUFFER_SIZE];
uint8_t spi1_rx_buffer[SPI_BUFFER_SIZE];

/* ----------------------------------------------------------------------------
 * Function      : void Initialize(void)
 * ----------------------------------------------------------------------------
 * Description   : Initialize the system by disabling interrupts, switching to
 *                 the 8 MHz clock (divided from the 48 MHz crystal) and
 *                 configuring the required DIOs to use the UART.
 * Inputs        : None
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void SPIInitialize(void)
{
//    /* Mask all interrupts */
//    __set_PRIMASK(PRIMASK_DISABLE_INTERRUPTS);
//
//    /* Disable all existing interrupts, clearing all pending source */
//    Sys_NVIC_DisableAllInt();
//    Sys_NVIC_ClearAllPendingInt();
//
//    /* Test DIO12 to pause the program to make it easy to re-flash */
//    DIO->CFG[RECOVERY_DIO] = DIO_MODE_INPUT  | DIO_WEAK_PULL_UP |
//                             DIO_LPF_DISABLE | DIO_6X_DRIVE;
//    while (DIO_DATA->ALIAS[RECOVERY_DIO] == 0);
//
//    /* Prepare the 48 MHz crystal
//     * Start and configure VDDRF */
//    ACS_VDDRF_CTRL->ENABLE_ALIAS = VDDRF_ENABLE_BITBAND;
//    ACS_VDDRF_CTRL->CLAMP_ALIAS  = VDDRF_DISABLE_HIZ_BITBAND;
//
//    /* Wait until VDDRF supply has powered up */
//    while (ACS_VDDRF_CTRL->READY_ALIAS != VDDRF_READY_BITBAND);
//
//    /* Enable RF power switches */
//    SYSCTRL_RF_POWER_CFG->RF_POWER_ALIAS   = RF_POWER_ENABLE_BITBAND;
//
//    /* Remove RF isolation */
//    SYSCTRL_RF_ACCESS_CFG->RF_ACCESS_ALIAS = RF_ACCESS_ENABLE_BITBAND;

//    /* Start the 48 MHz oscillator without changing the other register bits */
//    RF->XTAL_CTRL = ((RF->XTAL_CTRL & ~XTAL_CTRL_DISABLE_OSCILLATOR) |
//                     XTAL_CTRL_REG_VALUE_SEL_INTERNAL);

//    /* Enable 48 MHz oscillator divider at desired prescale value */
//    RF_REG2F->CK_DIV_1_6_CK_DIV_1_6_BYTE = CK_DIV_1_6_PRESCALE_1_BYTE;

//    /* Wait until 48 MHz oscillator is started */
//    while (RF_REG39->ANALOG_INFO_CLK_DIG_READY_ALIAS !=
//           ANALOG_INFO_CLK_DIG_READY_BITBAND);

//    /* Switch to (divided 48 MHz) oscillator clock */
//    Sys_Clocks_SystemClkConfig(JTCK_PRESCALE_1   |
//                               EXTCLK_PRESCALE_1 |
//                               SYSCLK_CLKSRC_RFCLK);

//    /* Setup DIO5 as a GPIO input with interrupts on transitions, DIO6 as a
//     * GPIO output. Use the integrated debounce circuit to ensure that only a
//     * single interrupt event occurs for each push of the pushbutton.
//     * The debounce circuit always has to be used in combination with the
//     * transition mode to deal with the debounce circuit limitations.
//     * A debounce filter time of 50 ms is used. */
//    Sys_DIO_Config(BUTTON_DIO, DIO_MODE_GPIO_IN_0 | DIO_WEAK_PULL_UP |
//                   DIO_LPF_DISABLE);
//    Sys_DIO_Config(LED_DIO, DIO_MODE_GPIO_OUT_0);
//    Sys_DIO_Config(RESET_DIO,DIO_MODE_GPIO_OUT_0);
//    Sys_DIO_IntConfig(0, DIO_EVENT_TRANSITION | DIO_SRC(BUTTON_DIO) |
//                      DIO_DEBOUNCE_ENABLE,
//                      DIO_DEBOUNCE_SLOWCLK_DIV1024, 49);

//    /* Enable BUTTON_DIO interrupt */
//    NVIC_EnableIRQ(DIO0_IRQn);
//
//    /* Stop masking interrupts */
//    __set_PRIMASK(PRIMASK_ENABLE_INTERRUPTS);
//    __set_FAULTMASK(FAULTMASK_ENABLE_INTERRUPTS);


	        /* Power-up SPI0 and register callback function  */
	        spi0 = &Driver_SPI0;
	        spi0->Initialize(SPI0_Master_CallBack);
	        spi0->PowerControl(ARM_POWER_FULL);


	        /* Power-up SPI1 and register callback function  */
	        spi1 = &Driver_SPI1;
	        spi1->Initialize(SPI1_Master_CallBack);
	        spi1->PowerControl(ARM_POWER_FULL);

	        //Configure AFE to desired register values and put in read mode on SPI0
	        Sys_GPIO_Set_Low(AFE_RESET);
	        Sys_Delay_ProgramROM(0.0005 * SystemCoreClock);
	        Sys_GPIO_Set_High(AFE_RESET);

	        AFEwrite2(0x00,0x000000);//put second AFE into write mode
	        AFEwrite(0x23,0x020500);//TRISTATE the second AFE to not interfere with AFE1
	        AFEtimerinit();
	        configAFE();
	        AFEreadmode();
	        AFEwrite2(0x23,0x020100);//undo the TRISTATE on second AFE

	        //Configure AFE to desired register values and put in read mode on SPI1//
	        AFEwrite(0x00,0x000000);//put first AFE into register write mode
	        AFEwrite(0x23,0x020500);//TRISTATE first AFE to not interfere with AFE2
	        AFEtimerinit2();
	        configAFE2();
	        AFEreadmode2();
	        AFEwrite(0x23,0x020100); //undo TRISTATE on first AFE so we can talk to it
	        AFEreadmode();
}

/* ----------------------------------------------------------------------------
 * Function      : void DIO0_IRQHandler(void)
 * ----------------------------------------------------------------------------
 * Description   : Interrupt handler triggered by a button press. Start an SPI
 *                 transfer.
 * Inputs        : None
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
//NOTE: I am not using the button press right now, since it was messing around

//void DIO12_IRQHandler(void)
//{
//    static uint8_t ignore_next_dio_int = 0;
//    if (ignore_next_dio_int)
//    {
//        ignore_next_dio_int = 0;
//    }
//    else if ((DIO_DATA->ALIAS[BUTTON_DIO] == 0) && (!spi0->GetStatus().busy))
//    {
//        /* Button is pressed: Ignore next interrupt.
//         * This is required to deal with the debounce circuit limitations. */
//        ignore_next_dio_int = 1;
//
//
//        /* Activate SSEL line and start transfer on SPI0/master */
//        //spi0->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_ACTIVE);
////        spi0->Transfer(SPI_TX_DATA,spi0_rx_buffer,  SPI_TRANSFER_COUNT);
//
//        //NEXT STEP: Setting up a loop within this interrupt to trigger readings
//        // execute writes on button press to make sure that we can run logic inside interrupts correctly
//        // get just two readings when the button is pressed
//        //then get the readings to happen for a set time and see if the readings make sense
//        //figure out timing between measurements and BLE stuff to maximize time spent in this loop
//        //hydration doesn't change that quickly, fewer measurements probably good enough
//
//
//
//
//    }
//}

/* ----------------------------------------------------------------------------
 * Function      : void ToggleGPIO(uint32_t dio, uint32_t n, float delay_ms)
 * ----------------------------------------------------------------------------
 * Description   : Toggle the specified GPIO 'n' times, delayed by 'delay_ms'
 *                 milliseconds.
 * Inputs        : dio       -    DIO number
 *                 n         -    Number of toggles
 *                 delay_ms  -    Delay between each toggle
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void ToggleGPIO(uint32_t dio, uint32_t n, uint32_t delay_ms)
{
    for (; n > 0; n--)
    {
        Sys_GPIO_Toggle(dio);
        Sys_Delay_ProgramROM((delay_ms / 1000.0) * SystemCoreClock);
    }
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI0_Master_CallBack(uint32_t event)
 * ----------------------------------------------------------------------------
 * Description   : This function is a callback registered by the function
 *                 Initialize, for SPI0. The parameter event indicates one or
 *                 more events that occurred during driver operation.
 * Inputs        : event    -    SPI Events notification mask
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */

//don't pay much attention to this, not sure if we will be using this for anything, really
void SPI0_Master_CallBack(uint32_t event)
{
    switch (event)
    {
        case ARM_SPI_EVENT_TRANSFER_COMPLETE:
        {
            /* Deactivate SSEL line */
            spi0->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_INACTIVE);
        }
        break;

        case ARM_SPI_EVENT_DATA_LOST:
        {
            /* Toggle LED state 10 times for 50 milliseconds */
            ToggleGPIO(LED_DIO, 10, 50);
        }
        break;
    }
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI1_Slave_CallBack(uint32_t event)
 * ----------------------------------------------------------------------------
 * Description   : This function is a callback registered by the function
 *                 Initialize, for SPI1. The parameter event indicates one or
 *                 more events that occurred during driver operation.
 * Inputs        : event    -    SPI Events notification mask
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */

//REALLY don't pay attention to this
void SPI1_Master_CallBack(uint32_t event)
{
    switch (event)
    {
        case ARM_SPI_EVENT_TRANSFER_COMPLETE:
        {
        				//chip enable goes high, SPI stops
        	            spi1->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_INACTIVE);
        }
        break;

        case ARM_SPI_EVENT_DATA_LOST:
        {
            /* Toggle LED state 10 times for 50 milliseconds */
            ToggleGPIO(LED_DIO, 10, 50);
        }
        break;
    }
}

/////////////////////Setting up SPI0 Interface Functions//////////////////////

//Function to send and receive the required bytes for our data formatting
void AFEtransfer(unsigned char address, unsigned int data, void* received){
	spi0->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
	                  ARM_SPI_SS_MASTER_SW, SPI_SPEED_BPS); //ensure SPI0 in active master mode
    unsigned char buffer[4]={address,data>>16,data>>8,data}; //initialize buffer for data
    spi0->Control(ARM_SPI_CONTROL_SS,ARM_SPI_SS_ACTIVE); //chip enable goes low, making SPI active
    spi0->Transfer(buffer,received,sizeof(buffer)); //transfer the data in buffer and receive data into "received"
    while(!ARM_SPI_EVENT_TRANSFER_COMPLETE); //wait for transfer to occur
    spi0->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_INACTIVE); //chip enable goes high, SPI ends
    Sys_Delay_ProgramROM(0.0005 * SystemCoreClock); //slight delay for timing
    spi0->Control(ARM_SPI_MODE_INACTIVE | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
                      ARM_SPI_SS_MASTER_SW, SPI_SPEED_BPS); //turn off SPI0 to not interefere with SPI1
}

//the god-SEND function that works thank goodness! (init timer registers, etc. for AFE)
void AFEwrite(unsigned char address, unsigned int data){
	spi0->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
	                  ARM_SPI_SS_MASTER_SW, SPI_SPEED_BPS); //ensure SPI0 in active master mode
    unsigned char buffer[4]={address,data>>16,data>>8,data}; //intialize buffer for data
    spi0->Control(ARM_SPI_CONTROL_SS,ARM_SPI_SS_ACTIVE); //Chip enable goes low, make SPI active
    spi0->Send(buffer,sizeof(buffer)); //send the buffer and ONLY the buffer
    while(!ARM_SPI_EVENT_TRANSFER_COMPLETE); //wait for sending to occur
    spi0->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_INACTIVE); //chip enable goes high, make SPI inactive
    Sys_Delay_ProgramROM(0.0005 * SystemCoreClock); //slight pause (may mess around with no pause)
    spi0->Control(ARM_SPI_MODE_INACTIVE | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
                      ARM_SPI_SS_MASTER_SW, SPI_SPEED_BPS);//turn off SPI0 to not interfere with SPI1
}

//configure timing registers for the AFE on SPI0 interface
void AFEtimerinit(void){
	AFEwrite(0x00, 0x000000); //write enable
	AFEwrite(0x01, 0x0017C0); //LED2STC = 6050
	AFEwrite(0x02, 0x001f3e); //LED2ENDC = 7998
	AFEwrite(0x03, 0x001770); //LED2LEDSTC = 6000
	AFEwrite(0x04, 0x001f3f); //LED2LEDENDC = 7999
	AFEwrite(0x05, 0x000050); //ALED2STC = 50
	AFEwrite(0x06, 0x0007ce); //ALED2ENDC = 1998
	AFEwrite(0x07, 0x000820); //LED1STC = 2050
	AFEwrite(0x08, 0x000f9e); //LED1ENDC = 3998
	AFEwrite(0x09, 0x0007d0); //LED1LEDSTC = 2000
	AFEwrite(0x0a, 0x000f9f); //LED1LEDENDC = 3999
	AFEwrite(0x0b, 0x000ff0); //ALED1STC = 4050
	AFEwrite(0x0c, 0x00176e); //ALED1ENDC = 5998
	AFEwrite(0x0d, 0x000006); //LED2CONVST = 4
	AFEwrite(0x0e, 0x0007cf); //LED2CONVEND = 1999
	AFEwrite(0x0f, 0x0007d6); //ALED2CONVST = 2004
	AFEwrite(0x10, 0x000f9f); //ALED2CONVEND = 3999
	AFEwrite(0x11, 0x000fa6); //LED1CONVST = 4004
	AFEwrite(0x12, 0x00176f); //LED1CONVEND = 5999
	AFEwrite(0x13, 0x001776); //ALED1CONVST = 6004
	AFEwrite(0x14, 0x001f3f); //ALED1CONVEND = 7999
	AFEwrite(0x15, 0x000000); //ADCRSTSTCT0 = 0
	AFEwrite(0x16, 0x000005); //ADCRSTENDCT0 = 3
	AFEwrite(0x17, 0x0007d0); //ADCRSTSTCT1 = 2000
	AFEwrite(0x18, 0x0007d5); //ADCRSTENDCT1 = 2003
	AFEwrite(0x19, 0x000fa0); //ADCRSTSTCT2 = 4000
	AFEwrite(0x1a, 0x000fa5); //ADCRSTENDCT2 = 4003
	AFEwrite(0x1b, 0x001770); //ADCRSTSTCT3 = 6000
	AFEwrite(0x1c, 0x001775); //ADCRSTENDCT3 = 6003
	AFEwrite(0x1d, 0x001f3f); //PRPCOUNT = 79994
}

//configure main control registers for the AFE on SPI0
void configAFE(void){
    AFEwrite(0x1e, 0x000101); //control1 register
    AFEwrite(0x20, 0x000000); // TIAGAIN
    AFEwrite(0x21, 0x000000); // TIAAMBGAIN
    AFEwrite(0x22, 0x011f1f); //LEDRANGE   /0x010f0f  6F is the current value for the first LED
    AFEwrite(0x23, 0x020100); //TXREF
}

//put the AFE into read mode to read off register values on SPI0
void AFEreadmode(void){
	AFEwrite(0x00,0x000001);
}

/////////////////////////////Setting up SPI1 Interface Functions///////////////////
////////////////////////////////////////////////////////////////////////////////

//function to initialize registers (only writes data, does not receive any information back)
void AFEwrite2(unsigned char address, unsigned int data){
	spi1->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
	                  ARM_SPI_SS_MASTER_SW, SPI_SPEED_BPS);// make sure SPI1 is active master
    unsigned char buffer[4]={address,data>>16,data>>8,data}; //intialize buffer for data
    spi1->Control(ARM_SPI_CONTROL_SS,ARM_SPI_SS_ACTIVE); //Chip enable goes low, make SPI active
    spi1->Send(buffer,sizeof(buffer)); //send the buffer and ONLY the buffer
    while(!ARM_SPI_EVENT_TRANSFER_COMPLETE); //wait for sending to occur
    spi1->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_INACTIVE); //chip enable goes high, make SPI inactive
    Sys_Delay_ProgramROM(0.0005 * SystemCoreClock); //slight pause (may mess around with no pause)
    spi1->Control(ARM_SPI_MODE_INACTIVE | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
                      ARM_SPI_SS_MASTER_SW, SPI_SPEED_BPS); //turn off SPI1 to not interfere with SPI0
}

//function to send and receive the required bytes for our data formatting from AFE4490
void AFEtransfer2(unsigned char address, unsigned int data, void* received){
	spi1->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
	                  ARM_SPI_SS_MASTER_SW, SPI_SPEED_BPS); //make sure SPI1 is active master
    unsigned char buffer[4]={address,data>>16,data>>8,data}; //initialize buffer for data tx
    spi1->Control(ARM_SPI_CONTROL_SS,ARM_SPI_SS_ACTIVE); //chip enable goes low, SPI1 active
    spi1->Transfer(buffer,received,sizeof(buffer)); //send and receive data
    while(!ARM_SPI_EVENT_TRANSFER_COMPLETE); //wait for transfer to complete
    spi1->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_INACTIVE); //chip enable goes high, SPI1 inactive
    Sys_Delay_ProgramROM(0.0005 * SystemCoreClock); //slight pause for timing
    spi1->Control(ARM_SPI_MODE_INACTIVE | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
                      ARM_SPI_SS_MASTER_SW, SPI_SPEED_BPS); //turn off SPI1 to not interfere with SPI0
}

//configure timing registers for the AFE on SPI1 interface
void AFEtimerinit2(void){
	AFEwrite2(0x00, 0x000000); //write enable
	AFEwrite2(0x01, 0x0017C0); //LED2STC = 6050
	AFEwrite2(0x02, 0x001f3e); //LED2ENDC = 7998
	AFEwrite2(0x03, 0x001770); //LED2LEDSTC = 6000
	AFEwrite2(0x04, 0x001f3f); //LED2LEDENDC = 7999
	AFEwrite2(0x05, 0x000050); //ALED2STC = 50
	AFEwrite2(0x06, 0x0007ce); //ALED2ENDC = 1998
	AFEwrite2(0x07, 0x000820); //LED1STC = 2050
	AFEwrite2(0x08, 0x000f9e); //LED1ENDC = 3998
	AFEwrite2(0x09, 0x0007d0); //LED1LEDSTC = 2000
	AFEwrite2(0x0a, 0x000f9f); //LED1LEDENDC = 3999
	AFEwrite2(0x0b, 0x000ff0); //ALED1STC = 4050
	AFEwrite2(0x0c, 0x00176e); //ALED1ENDC = 5998
	AFEwrite2(0x0d, 0x000006); //LED2CONVST = 4
	AFEwrite2(0x0e, 0x0007cf); //LED2CONVEND = 1999
	AFEwrite2(0x0f, 0x0007d6); //ALED2CONVST = 2004
	AFEwrite2(0x10, 0x000f9f); //ALED2CONVEND = 3999
	AFEwrite2(0x11, 0x000fa6); //LED1CONVST = 4004
	AFEwrite2(0x12, 0x00176f); //LED1CONVEND = 5999
	AFEwrite2(0x13, 0x001776); //ALED1CONVST = 6004
	AFEwrite2(0x14, 0x001f3f); //ALED1CONVEND = 7999
	AFEwrite2(0x15, 0x000000); //ADCRSTSTCT0 = 0
	AFEwrite2(0x16, 0x000005); //ADCRSTENDCT0 = 3
	AFEwrite2(0x17, 0x0007d0); //ADCRSTSTCT1 = 2000
	AFEwrite2(0x18, 0x0007d5); //ADCRSTENDCT1 = 2003
	AFEwrite2(0x19, 0x000fa0); //ADCRSTSTCT2 = 4000
	AFEwrite2(0x1a, 0x000fa5); //ADCRSTENDCT2 = 4003
	AFEwrite2(0x1b, 0x001770); //ADCRSTSTCT3 = 6000
	AFEwrite2(0x1c, 0x001775); //ADCRSTENDCT3 = 6003
	AFEwrite2(0x1d, 0x001f3f); //PRPCOUNT = 79994
}

//configure main control registers for the AFE on SPI1
void configAFE2(void){
    AFEwrite2(0x1e, 0x000101); //control1 register
    AFEwrite2(0x20, 0x000000); // TIAGAIN
    AFEwrite2(0x21, 0x000000); // TIAAMBGAIN
    AFEwrite2(0x22, 0x011f1f); //LEDRANGE   //first 0F is first led current value, second is the second LED current value
    AFEwrite2(0x23, 0x020100); //TXREF
}

//put the AFE into read mode to read off register values on SPI1
void AFEreadmode2(void){
	AFEwrite2(0x00,0x000001);
}

