/*
 * SPI.h
 *
 *  Created on: Mar 26, 2019
 *      Author: rhoehne
 */

#ifndef INCLUDE_SPI_H_
#define INCLUDE_SPI_H_

/* ----------------------------------------------------------------------------
 * If building with a C++ compiler, make all of the definitions in this header
 * have a C binding.
 * ------------------------------------------------------------------------- */
#ifdef __cplusplus
extern "C"
{
#endif    /* ifdef __cplusplus */

/* ----------------------------------------------------------------------------
 * Include files
 * --------------------------------------------------------------------------*/
#include <rsl10.h>
#include <string.h>
#include <stdio.h>
#include "SPI_RSLxx.h"
#include "RTE_Device.h"

/* ----------------------------------------------------------------------------
 * Defines
 * --------------------------------------------------------------------------*/
#if (!RTE_SPI0 || !RTE_SPI1)
    #error "Please configure SPI in RTE_Device.h"
#endif    /* if (!RTE_SPI0 || !RTE_SPI1) */

#define CONCAT(x, y)                    x##y
#define DIO_SRC(x)                      CONCAT(DIO_SRC_DIO_, x)

/* DIO number that is used for easy re-flashing (recovery mode) */
#define RECOVERY_DIO                    11

/* GPIO definitions */
#define LED_DIO                         1
#define BUTTON_DIO                      12
#define AFE_RESET						4
#define AFE_PDN							2


// set current at 50mA

/* SPI related definitions. SPI TX/RX pins are configured in RTE_Device.h */
#define SPI_TX_DATA                    "RSL"
#define SPI_BUFFER_SIZE                8
#define SPI_SPEED_BPS                  100000 //100000
#define SPI_DATA_BITS                  8    /* Options: 8, 16, 24 or 32 */
#define SPI_TRANSFER_COUNT             (SPI_BUFFER_SIZE / (SPI_DATA_BITS / 8))

/* ----------------------------------------------------------------------------
 * Global variables and types
 * --------------------------------------------------------------------------*/
extern ARM_DRIVER_SPI Driver_SPI0, Driver_SPI1;
extern ARM_DRIVER_SPI *spi0, *spi1;
extern uint8_t spi0_rx_buffer[SPI_BUFFER_SIZE];
extern uint8_t spi1_rx_buffer[SPI_BUFFER_SIZE];

/* ---------------------------------------------------------------------------
* Function prototype definitions
* --------------------------------------------------------------------------*/
extern void SPI0_Master_CallBack(uint32_t event);

extern void SPI1_Master_CallBack(uint32_t event);

extern void ToggleGPIO(uint32_t dio, uint32_t n, uint32_t delay_ms);

extern void DIO12_IRQHandler(void);

extern void SPIInitialize(void);

extern void AFEtransfer(unsigned char address, unsigned int data, void* received);

extern void AFEwrite(unsigned char address, unsigned int data);

extern void AFEtimerinit(void);

extern void configAFE(void);

extern void AFEreadmode(void);

extern void AFEtransfer2(unsigned char address, unsigned int data, void* received);

extern void AFEwrite2(unsigned char address, unsigned int data);

extern void AFEtimerinit2(void);

extern void configAFE2(void);

extern void AFEreadmode2(void);

/* ----------------------------------------------------------------------------
 * Close the 'extern "C"' block
 * ------------------------------------------------------------------------- */
#ifdef __cplusplus
}
#endif    /* ifdef __cplusplus */


#endif /* INCLUDE_SPI_H_ */
